@@ -26,13 +26,12 @@
 #include "svn_error.h"
 #include "svn_opt.h"
 #include "svn_utf.h"
-#include "svn_subst.h"
 #include "svn_path.h"
-#include "svn_config.h"
 #include "svn_hash.h"
 #include "svn_repos.h"
 #include "svn_pools.h"
 #include "svn_sorts.h"
+#include "svn_props.h"
 
 
 /*** Code. ***/
@@ -59,7 +58,7 @@ create_stdio_stream (svn_stream_t **stream,
   apr_status_t apr_err = open_fn (&stdio_file, pool);
 
   if (apr_err)
-    return svn_error_wrap_apr (apr_err, "Can't open stdio file");
+    return svn_error_wrap_apr (apr_err, _("Can't open stdio file"));
 
   *stream = svn_stream_from_aprfile (stdio_file, pool);
   return SVN_NO_ERROR;
@@ -322,8 +321,8 @@ output_revision (struct revision_baton_t *rb)
                     apr_hash_get (old_props, SVN_PROP_REVISION_DATE, 
                                   APR_HASH_KEY_STRING));
       apr_hash_set (rb->props, SVN_PROP_REVISION_LOG, APR_HASH_KEY_STRING,
-                    svn_string_create ("This is an empty revision for "
-                                       "padding.", hash_pool));
+                    svn_string_create (_("This is an empty revision for "
+                                         "padding."), hash_pool));
     }
 
   /* Now, "rasterize" the props to a string, and append the property
@@ -529,7 +528,7 @@ new_node_record (void **node_baton,
                   /* bail out with an error */
                   return svn_error_createf
                     (SVN_ERR_NODE_UNEXPECTED_KIND, NULL,
-                     "Node with dropped parent sneaked in");
+                     _("Node with dropped parent sneaked in"));
                 }
               SVN_ERR (svn_stream_printf
                        (nb->rb->pb->out_stream, pool,
@@ -680,7 +679,7 @@ static svn_error_t *
 close_node (void *node_baton)
 {
   struct node_baton_t *nb = node_baton;
-  int len = 2;
+  apr_size_t len = 2;
 
   /* Get out of here if we can. */
   if (nb->do_skip)
@@ -753,21 +752,21 @@ enum
 static const apr_getopt_option_t options_table[] =
   {
     {"help",          'h', 0,
-     "show help on a subcommand"},
+     N_("show help on a subcommand")},
 
     {NULL,            '?', 0,
-     "show help on a subcommand"},
+     N_("show help on a subcommand")},
 
     {"version",            svndumpfilter__version, 0,
-     "show version information" },
+     N_("show version information") },
     {"quiet",              svndumpfilter__quiet, 0,
-     "Do not display filtering statistics." },
+     N_("Do not display filtering statistics.") },
     {"drop-empty-revs",    svndumpfilter__drop_empty_revs, 0,
-     "Remove revisions emptied by filtering."},
+     N_("Remove revisions emptied by filtering.")},
     {"renumber-revs",      svndumpfilter__renumber_revs, 0,
-     "Renumber revisions left after filtering." },
+     N_("Renumber revisions left after filtering.") },
     {"preserve-revprops",  svndumpfilter__preserve_revprops, 0,
-     "Don't filter revision properties." },
+     N_("Don't filter revision properties.") },
     {NULL}
   };
 
@@ -778,20 +777,20 @@ static const apr_getopt_option_t options_table[] =
 static const svn_opt_subcommand_desc_t cmd_table[] =
   {
     {"exclude", subcommand_exclude, {0},
-     "Filter out nodes with given prefixes from dumpstream.\n"
-     "usage: svndumpfilter exclude PATH_PREFIX...\n",
+     N_("Filter out nodes with given prefixes from dumpstream.\n"
+        "usage: svndumpfilter exclude PATH_PREFIX...\n"),
      {svndumpfilter__drop_empty_revs, svndumpfilter__renumber_revs,
       svndumpfilter__preserve_revprops, svndumpfilter__quiet} },
 
     {"include", subcommand_include, {0},
-     "Filter out nodes without given prefixes from dumpstream.\n"
-     "usage: svndumpfilter include PATH_PREFIX...\n",
+     N_("Filter out nodes without given prefixes from dumpstream.\n"
+        "usage: svndumpfilter include PATH_PREFIX...\n"),
      {svndumpfilter__drop_empty_revs, svndumpfilter__renumber_revs,
       svndumpfilter__preserve_revprops, svndumpfilter__quiet} },
 
     {"help", subcommand_help, {"?", "h"},
-     "Describe the usage of this program or its subcommands.\n"
-     "usage: svndumpfilter help [SUBCOMMAND...]\n",
+     N_("Describe the usage of this program or its subcommands.\n"
+        "usage: svndumpfilter help [SUBCOMMAND...]\n"),
      {svndumpfilter__version} },
 
     { NULL, NULL, {0}, NULL, {0} }
@@ -860,11 +859,11 @@ subcommand_help (apr_getopt_t *os, void *baton, apr_pool_t *pool)
 {
   struct svndumpfilter_opt_state *opt_state = baton;
   const char *header =
-    "general usage: svndumpfilter SUBCOMMAND [ARGS & OPTIONS ...]\n"
-    "Type \"svndumpfilter help <subcommand>\" for help on a "
-    "specific subcommand.\n"
-    "\n"
-    "Available subcommands:\n";
+    _("general usage: svndumpfilter SUBCOMMAND [ARGS & OPTIONS ...]\n"
+      "Type 'svndumpfilter help <subcommand>' for help on a "
+      "specific subcommand.\n"
+      "\n"
+      "Available subcommands:\n");
 
   SVN_ERR (svn_opt_print_help (os, "svndumpfilter",
                                opt_state ? opt_state->version : FALSE,
@@ -884,13 +883,12 @@ check_lib_versions (void)
     {
       { "svn_subr",  svn_subr_version },
       { "svn_repos", svn_repos_version },
-      { "svn_fs",    svn_fs_version },
       { "svn_delta", svn_delta_version },
       { NULL, NULL }
     };
 
-   SVN_VERSION_DEFINE (my_version);
-   return svn_ver_check_list (&my_version, checklist);
+  SVN_VERSION_DEFINE (my_version);
+  return svn_ver_check_list (&my_version, checklist);
 }
 
 
@@ -913,11 +911,15 @@ do_filter (apr_getopt_t *os,
       apr_pool_t *subpool = svn_pool_create (pool);
 
       SVN_ERR (svn_cmdline_fprintf (stderr, subpool,
-                                    "%s %sprefixes:\n",
-                                    do_exclude ? "Excluding" : "Including",
-                                    opt_state->drop_empty_revs 
-                                    ? "(and dropping empty revisions for) "
-                                    : ""));
+                                    do_exclude
+                                    ? opt_state->drop_empty_revs
+                                      ? _("Excluding (and dropping empty "
+                                          "revisions for) prefixes:\n")
+                                      : _("Excluding prefixes:\n")
+                                    : opt_state->drop_empty_revs
+                                      ? _("Including (and dropping empty "
+                                          "revisions for) prefixes:\n")
+                                      : _("Including prefixes:\n")));
 
       for (i = 0; i < opt_state->prefixes->nelts; i++)
         {
@@ -962,7 +964,7 @@ do_filter (apr_getopt_t *os,
            hi = apr_hash_next (hi))
         {
           apr_hash_this (hi, &key, NULL, NULL);
-          APR_ARRAY_PUSH (keys, svn_revnum_t) = *((svn_revnum_t *) key);
+          APR_ARRAY_PUSH (keys, svn_revnum_t) = *((const svn_revnum_t *) key);
         }
       qsort (keys->elts, keys->nelts, 
              keys->elt_size, svn_sort_compare_revisions);
@@ -982,7 +984,7 @@ do_filter (apr_getopt_t *os,
                                           this_key));
           else
             SVN_ERR (svn_cmdline_fprintf (stderr, subpool,
-                                          _("   %ld => %ld\n"),
+                                          "   %ld => %ld\n",
                                           this_key, this_val));
         }
       SVN_ERR (svn_cmdline_fputs ("\n", stderr, subpool));
@@ -1076,7 +1078,7 @@ main (int argc, const char * const *argv)
   err = check_lib_versions ();
   if (err)
     {
-      svn_handle_error (err, stderr, FALSE);
+      svn_handle_error2 (err, stderr, FALSE, "svndumpfilter: ");
       svn_error_clear (err);
       svn_pool_destroy (pool);
       return EXIT_FAILURE;
@@ -1160,7 +1162,7 @@ main (int argc, const char * const *argv)
       if (os->ind >= os->argc)
         {
           svn_error_clear (svn_cmdline_fprintf
-                           (stderr, pool, "subcommand argument required\n"));
+                           (stderr, pool, _("Subcommand argument required\n")));
           subcommand_help (NULL, NULL, pool);
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
@@ -1175,14 +1177,14 @@ main (int argc, const char * const *argv)
               if ((err = svn_utf_cstring_to_utf8 (&first_arg_utf8, first_arg,
                                                   pool)))
                 {
-                  svn_handle_error (err, stderr, FALSE);
+                  svn_handle_error2 (err, stderr, FALSE, "svndumpfilter: ");
                   svn_pool_destroy (pool);
                   svn_error_clear (err);
                   return EXIT_FAILURE;
                 }
                 
               svn_error_clear (svn_cmdline_fprintf (stderr, pool,
-                                                    "unknown command: '%s'\n",
+                                                    _("Unknown command: '%s'\n"),
                                                     first_arg_utf8));
               subcommand_help (NULL, NULL, pool);
               svn_pool_destroy (pool);
@@ -1244,7 +1246,7 @@ main (int argc, const char * const *argv)
           svn_opt_format_option (&optstr, badopt, FALSE, pool);
           svn_error_clear (svn_cmdline_fprintf
                            (stderr, pool,
-                            _("subcommand '%s' doesn't accept option '%s'\n"
+                            _("Subcommand '%s' doesn't accept option '%s'\n"
                               "Type 'svndumpfilter help %s' for usage.\n"),
                             subcommand->name, optstr, subcommand->name));
           svn_pool_destroy (pool);
@@ -1258,13 +1260,13 @@ main (int argc, const char * const *argv)
     {
       if (err->apr_err == SVN_ERR_CL_ARG_PARSING_ERROR)
         {
-          svn_handle_error (err, stderr, 0);
+          svn_handle_error2 (err, stderr, FALSE, "svndumpfilter: ");
           svn_opt_subcommand_help (subcommand->name, cmd_table,
                                    options_table, pool);
         }
       else
         {
-          svn_handle_error (err, stderr, 0);
+          svn_handle_error2 (err, stderr, FALSE, "svndumpfilter: ");
         }
       svn_pool_destroy (pool);
       return EXIT_FAILURE;
