@@ -177,10 +177,10 @@ def basic_copy_and_move_files(sbox):
   expected_status.tweak('A/D/G/rho', 'A/mu', wc_rev=2)
 
   expected_status.add({
-    'A/D/rho' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/C/alpha2' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/D/H/mu' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/F/iota' : Item(status='  ', wc_rev=2, repos_rev=2),
+    'A/D/rho' : Item(status='  ', wc_rev=2),
+    'A/C/alpha2' : Item(status='  ', wc_rev=2),
+    'A/D/H/mu' : Item(status='  ', wc_rev=2),
+    'A/B/F/iota' : Item(status='  ', wc_rev=2),
     })
 
   expected_status.remove('A/mu', 'iota')
@@ -200,7 +200,6 @@ def basic_copy_and_move_files(sbox):
     'A/C/alpha2' : Item(verb='Sending'),
     })
   expected_status.tweak('A/C/alpha2', wc_rev=3)
-  expected_status.tweak(repos_rev=3)
   svntest.actions.run_and_verify_commit (wc_dir,
                                          expected_output,
                                          expected_status,
@@ -319,10 +318,10 @@ def receive_copy_in_update(sbox):
   expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
   expected_status.tweak(wc_rev=1)
   expected_status.add({
-    'A/B/newG' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/newG/pi' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/newG/rho' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/newG/tau' : Item(status='  ', wc_rev=2, repos_rev=2),
+    'A/B/newG' : Item(status='  ', wc_rev=2),
+    'A/B/newG/pi' : Item(status='  ', wc_rev=2),
+    'A/B/newG/rho' : Item(status='  ', wc_rev=2),
+    'A/B/newG/tau' : Item(status='  ', wc_rev=2),
     })
 
   svntest.actions.run_and_verify_commit (wc_dir,
@@ -356,10 +355,10 @@ def receive_copy_in_update(sbox):
   # Create expected status tree for the update.
   expected_status = svntest.actions.get_virginal_state(wc_backup, 2)
   expected_status.add({
-    'A/B/newG' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/newG/pi' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/newG/rho' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/newG/tau' : Item(status='  ', wc_rev=2, repos_rev=2),
+    'A/B/newG' : Item(status='  ', wc_rev=2),
+    'A/B/newG/pi' : Item(status='  ', wc_rev=2),
+    'A/B/newG/rho' : Item(status='  ', wc_rev=2),
+    'A/B/newG/tau' : Item(status='  ', wc_rev=2),
     })
 
   # Do the update and check the results in three ways.
@@ -395,7 +394,6 @@ def resurrect_deleted_dir(sbox):
     })
 
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
-  expected_status.tweak(repos_rev=2)
   expected_status.remove('A/D/G')
   expected_status.remove('A/D/G/pi')
   expected_status.remove('A/D/G/rho')
@@ -575,7 +573,6 @@ def copy_files_with_properties(sbox):
     'A/D/G/rho' : Item(verb='Sending'),
     })
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
-  expected_status.tweak(repos_rev=2)
   expected_status.tweak('A/D/G/rho', status='  ', wc_rev=2)
   svntest.actions.run_and_verify_commit(wc_dir,
                                         expected_output, expected_status,
@@ -599,9 +596,9 @@ def copy_files_with_properties(sbox):
 
   # Properties are not visible in WC status 'A'
   expected_status.add({
-    'A/D/G/rho' : Item(status=' M', wc_rev='2', repos_rev='2'),
-    'A/D/G/rho_wc' : Item(status='A ', wc_rev='-', repos_rev='2', copied='+'),
-    'A/D/G/rho_url' : Item(status='A ', wc_rev='-', repos_rev='2', copied='+'),
+    'A/D/G/rho' : Item(status=' M', wc_rev='2'),
+    'A/D/G/rho_wc' : Item(status='A ', wc_rev='-', copied='+'),
+    'A/D/G/rho_url' : Item(status='A ', wc_rev='-', copied='+'),
     })
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
@@ -619,12 +616,11 @@ def copy_files_with_properties(sbox):
     'A/D/G/rho_wc' : Item(verb='Adding'),
     'A/D/G/rho_url' : Item(verb='Adding'),
     })
-  expected_status.tweak(repos_rev=3)
   expected_status.tweak('A/D/G/rho', status='  ', wc_rev=3)
   expected_status.remove('A/D/G/rho_wc', 'A/D/G/rho_url')
   expected_status.add({
-    'A/D/G/rho_wc' : Item(status='  ', wc_rev=3, repos_rev=3),
-    'A/D/G/rho_url' : Item(status='  ', wc_rev=3, repos_rev=3),
+    'A/D/G/rho_wc' : Item(status='  ', wc_rev=3),
+    'A/D/G/rho_url' : Item(status='  ', wc_rev=3),
     })
   svntest.actions.run_and_verify_commit(wc_dir,
                                         expected_output, expected_status,
@@ -699,9 +695,9 @@ def mv_and_revert_directory(sbox):
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_status.tweak('A/B/E', 'A/B/E/alpha', 'A/B/E/beta', status='D ')
   expected_status.add({
-    'A/B/F/E' : Item(status='A ', wc_rev='-', repos_rev='1', copied='+'),
-    'A/B/F/E/alpha' : Item(status='  ', wc_rev='-', repos_rev='1', copied='+'),
-    'A/B/F/E/beta' : Item(status='  ', wc_rev='-', repos_rev='1', copied='+'),
+    'A/B/F/E' : Item(status='A ', wc_rev='-', copied='+'),
+    'A/B/F/E/alpha' : Item(status='  ', wc_rev='-', copied='+'),
+    'A/B/F/E/beta' : Item(status='  ', wc_rev='-', copied='+'),
     })
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
@@ -808,14 +804,14 @@ def wc_to_repos(sbox):
     })
   expected_status = svntest.actions.get_virginal_state(wc_dir, 4)
   expected_status.add({
-    'A/B/E/beta'   : Item(status=' M', wc_rev=4, repos_rev=4),
-    'A/D/H/omega'  : Item(status='M ', wc_rev=4, repos_rev=4),
-    'A/B/E/beta2'  : Item(status='  ', wc_rev=4, repos_rev=4),
-    'A/D/H2'       : Item(status='  ', wc_rev=4, repos_rev=4),
-    'A/D/H2/chi'   : Item(status='  ', wc_rev=4, repos_rev=4),
-    'A/D/H2/omega' : Item(status='  ', wc_rev=4, repos_rev=4),
-    'A/D/H2/psi'   : Item(status='  ', wc_rev=4, repos_rev=4),
-    'A/D/H2/beta'  : Item(status='  ', wc_rev=4, repos_rev=4),
+    'A/B/E/beta'   : Item(status=' M', wc_rev=4),
+    'A/D/H/omega'  : Item(status='M ', wc_rev=4),
+    'A/B/E/beta2'  : Item(status='  ', wc_rev=4),
+    'A/D/H2'       : Item(status='  ', wc_rev=4),
+    'A/D/H2/chi'   : Item(status='  ', wc_rev=4),
+    'A/D/H2/omega' : Item(status='  ', wc_rev=4),
+    'A/D/H2/psi'   : Item(status='  ', wc_rev=4),
+    'A/D/H2/beta'  : Item(status='  ', wc_rev=4),
     })
   svntest.actions.run_and_verify_update(wc_dir,
                                         expected_output,
@@ -848,18 +844,34 @@ def repos_to_wc(sbox):
   # we should get some scheduled additions *with history*.
   E_url = svntest.main.current_repo_url + "/A/B/E"
   pi_url = svntest.main.current_repo_url + "/A/D/G/pi"
+  pi_path = os.path.join (wc_dir, 'pi')
 
   svntest.actions.run_and_verify_svn(None, None, [], 'copy', E_url, wc_dir)
   svntest.actions.run_and_verify_svn(None, None, [], 'copy', pi_url, wc_dir)
 
+  # Extra test: modify file ASAP to check there was a timestamp sleep
+  svntest.main.file_append(pi_path, 'zig')
+
   expected_output = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_output.add({
-    'pi' : Item(status='A ', copied='+', wc_rev='-', repos_rev=1),
-    'E' :  Item(status='A ', copied='+', wc_rev='-', repos_rev=1),
-    'E/alpha' :  Item(status='  ', copied='+', wc_rev='-', repos_rev=1),
-    'E/beta'  :  Item(status='  ', copied='+', wc_rev='-', repos_rev=1),
+    'pi' : Item(status='A ', copied='+', wc_rev='-'),
+    'E' :  Item(status='A ', copied='+', wc_rev='-'),
+    'E/alpha' :  Item(status='  ', copied='+', wc_rev='-'),
+    'E/beta'  :  Item(status='  ', copied='+', wc_rev='-'),
     })
   svntest.actions.run_and_verify_status (wc_dir, expected_output)
+
+  # Modification will only show up if timestamps differ
+  out,err = svntest.main.run_svn(None, 'diff', pi_path)
+  if err or not out:
+    print "diff failed"
+    raise svntest.Failure
+  for line in out:
+    if line == '-This is the file \'pi\'.\n': # Crude check for diff-like output
+      break
+  else:
+    print "diff output incorrect", out
+    raise svntest.Failure
   
   # Revert everything and verify.
   svntest.actions.run_and_verify_svn(None, None, [], 'revert', '-R', wc_dir)
@@ -878,7 +890,7 @@ def repos_to_wc(sbox):
 
   expected_output = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_output.add({
-    'C' :  Item(status='A ', copied='+', wc_rev='-', repos_rev=1),
+    'C' :  Item(status='A ', copied='+', wc_rev='-'),
     })
   svntest.actions.run_and_verify_status (wc_dir, expected_output)
   
@@ -905,7 +917,7 @@ def repos_to_wc(sbox):
 
   expected_output = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_output.add({
-    'pi' : Item(status='A ',  wc_rev='0', repos_rev=1),
+    'pi' : Item(status='A ',  wc_rev='0'),
     })
   svntest.actions.run_and_verify_status (wc_dir, expected_output)
 
@@ -966,12 +978,12 @@ def url_copy_parent_into_child(sbox):
     })
   expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
   expected_status.add({
-    'A/B/F/B'         : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/F/B/E'       : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/F/B/E/alpha' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/F/B/E/beta'  : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/F/B/F'       : Item(status='  ', wc_rev=2, repos_rev=2),
-    'A/B/F/B/lambda'  : Item(status='  ', wc_rev=2, repos_rev=2),
+    'A/B/F/B'         : Item(status='  ', wc_rev=2),
+    'A/B/F/B/E'       : Item(status='  ', wc_rev=2),
+    'A/B/F/B/E/alpha' : Item(status='  ', wc_rev=2),
+    'A/B/F/B/E/beta'  : Item(status='  ', wc_rev=2),
+    'A/B/F/B/F'       : Item(status='  ', wc_rev=2),
+    'A/B/F/B/lambda'  : Item(status='  ', wc_rev=2),
     })
   svntest.actions.run_and_verify_update(wc_dir,
                                         expected_output,
@@ -1035,18 +1047,18 @@ def wc_copy_parent_into_child(sbox):
     'F/B/lambda'  : Item("This is the file 'lambda'."),
     })
   expected_status = svntest.wc.State(wc_dir, {
-    ''            : Item(status='  ', wc_rev=2, repos_rev=2),
-    'E'           : Item(status='  ', wc_rev=2, repos_rev=2),
-    'E/alpha'     : Item(status='  ', wc_rev=2, repos_rev=2),
-    'E/beta'      : Item(status='  ', wc_rev=2, repos_rev=2),
-    'F'           : Item(status='  ', wc_rev=2, repos_rev=2),
-    'lambda'      : Item(status='  ', wc_rev=2, repos_rev=2),
-    'F/B'         : Item(status='  ', wc_rev=2, repos_rev=2),
-    'F/B/E'       : Item(status='  ', wc_rev=2, repos_rev=2),
-    'F/B/E/alpha' : Item(status='  ', wc_rev=2, repos_rev=2),
-    'F/B/E/beta'  : Item(status='  ', wc_rev=2, repos_rev=2),
-    'F/B/F'       : Item(status='  ', wc_rev=2, repos_rev=2),
-    'F/B/lambda'  : Item(status='  ', wc_rev=2, repos_rev=2),
+    ''            : Item(status='  ', wc_rev=2),
+    'E'           : Item(status='  ', wc_rev=2),
+    'E/alpha'     : Item(status='  ', wc_rev=2),
+    'E/beta'      : Item(status='  ', wc_rev=2),
+    'F'           : Item(status='  ', wc_rev=2),
+    'lambda'      : Item(status='  ', wc_rev=2),
+    'F/B'         : Item(status='  ', wc_rev=2),
+    'F/B/E'       : Item(status='  ', wc_rev=2),
+    'F/B/E/alpha' : Item(status='  ', wc_rev=2),
+    'F/B/E/beta'  : Item(status='  ', wc_rev=2),
+    'F/B/F'       : Item(status='  ', wc_rev=2),
+    'F/B/lambda'  : Item(status='  ', wc_rev=2),
     })
   svntest.actions.run_and_verify_update(wc_dir,
                                         expected_output,
@@ -1088,7 +1100,7 @@ def resurrect_deleted_file(sbox):
 
   # status should now show the file scheduled for addition-with-history
   expected_status.add({
-    'rho' : Item(status='A ', copied='+', wc_rev='-', repos_rev=2),
+    'rho' : Item(status='A ', copied='+', wc_rev='-'),
     })
   svntest.actions.run_and_verify_status (wc_dir, expected_status)
 
@@ -1264,13 +1276,7 @@ def copy_over_missing_file(sbox):
   # Make sure that the working copy is not corrupted:
   expected_disk = svntest.main.greek_state.copy()
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
-  # Technically, we should expect "Restored" for A/mu, but apparently
-  # our output tree constructor doesn't know how to handle that, so we
-  # just pass an empty dictionary below.  It doesn't matter so much,
-  # because the bug we're testing for would leave a bogus logfile in
-  # the .svn/ area after the copy; subsequent updates would fail in a
-  # much more dramatic way than merely not matching expected output.
-  expected_output = svntest.wc.State(wc_dir, {})
+  expected_output = svntest.wc.State(wc_dir, {'A/mu' : Item(verb='Restored')})
   svntest.actions.run_and_verify_update(wc_dir,
                                         expected_output,
                                         expected_disk,
@@ -1295,7 +1301,6 @@ def repos_to_wc_1634(sbox):
     })
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_status.remove('A/B/E', 'A/B/E/alpha', 'A/B/E/beta')
-  expected_status.tweak(repos_rev=2)
   svntest.actions.run_and_verify_commit (wc_dir,
                                          expected_output,
                                          expected_status,
@@ -1307,18 +1312,18 @@ def repos_to_wc_1634(sbox):
   svntest.actions.run_and_verify_svn(None, None, [],
                                      'copy', '-r1', E_url, E_path)
   expected_status.add({
-    'A/B/E'       :  Item(status='A ', copied='+', wc_rev='-', repos_rev=2),
-    'A/B/E/alpha' :  Item(status='  ', copied='+', wc_rev='-', repos_rev=2),
-    'A/B/E/beta'  :  Item(status='  ', copied='+', wc_rev='-', repos_rev=2),
+    'A/B/E'       :  Item(status='A ', copied='+', wc_rev='-'),
+    'A/B/E/alpha' :  Item(status='  ', copied='+', wc_rev='-'),
+    'A/B/E/beta'  :  Item(status='  ', copied='+', wc_rev='-'),
     })
   svntest.actions.run_and_verify_status (wc_dir, expected_status)
 
   svntest.actions.run_and_verify_svn(None, None, [], 'up', wc_dir)
   expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
   expected_status.add({
-    'A/B/E'       :  Item(status='A ', copied='+', wc_rev='-', repos_rev=2),
-    'A/B/E/alpha' :  Item(status='  ', copied='+', wc_rev='-', repos_rev=2),
-    'A/B/E/beta'  :  Item(status='  ', copied='+', wc_rev='-', repos_rev=2),
+    'A/B/E'       :  Item(status='A ', copied='+', wc_rev='-'),
+    'A/B/E/alpha' :  Item(status='  ', copied='+', wc_rev='-'),
+    'A/B/E/beta'  :  Item(status='  ', copied='+', wc_rev='-'),
     })
   svntest.actions.run_and_verify_status (wc_dir, expected_status)
 
@@ -1354,6 +1359,208 @@ def double_uri_escaping_1814(sbox):
                                      '-R', base_url)
 
 
+#----------------------------------------------------------------------
+#  Regression test for issues 2101 and 2020
+
+def wc_to_wc_copy_deleted(sbox):
+  "wc to wc copy with deleted=true items"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  B_path = os.path.join(wc_dir, 'A', 'B')
+  B2_path = os.path.join(wc_dir, 'A', 'B2')
+
+  # Schedule for delete
+  svntest.actions.run_and_verify_svn(None, None, [], 'rm',
+                                     os.path.join(B_path, 'E', 'alpha'),
+                                     os.path.join(B_path, 'lambda'),
+                                     os.path.join(B_path, 'F'))
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
+  expected_status.tweak('A/B/E/alpha', 'A/B/lambda', 'A/B/F', status='D ')
+  svntest.actions.run_and_verify_status (wc_dir, expected_status)
+
+  # Copy to schedule=delete fails
+  out, err = svntest.main.run_svn(1, 'cp',
+                                  os.path.join(B_path, 'E'),
+                                  os.path.join(B_path, 'F'))
+  for line in err:
+    if line.find("is scheduled for deletion") != -1:
+      break
+  else:
+    raise svntest.Failure
+  svntest.actions.run_and_verify_status (wc_dir, expected_status)
+
+
+  # Commit to get state deleted
+  expected_status.remove('A/B/E/alpha', 'A/B/lambda', 'A/B/F')
+  expected_output = svntest.wc.State(wc_dir, {
+    'A/B/E/alpha' : Item(verb='Deleting'),
+    'A/B/lambda'  : Item(verb='Deleting'),
+    'A/B/F'       : Item(verb='Deleting'),
+    })
+  svntest.actions.run_and_verify_commit (wc_dir,
+                                         expected_output,
+                                         expected_status,
+                                         None, None, None, None, None,
+                                         wc_dir)
+
+  # Copy including stuff in state deleted=true
+  svntest.actions.run_and_verify_svn(None, None, [], 'copy', B_path, B2_path)
+  expected_status.add({
+    'A/B2'         : Item(status='A ', wc_rev='-', copied='+'),
+    'A/B2/E'       : Item(status='  ', wc_rev='-', copied='+'),
+    'A/B2/E/beta'  : Item(status='  ', wc_rev='-', copied='+'),
+    'A/B2/E/alpha' : Item(status='D ', wc_rev='-', copied='+'),
+    'A/B2/lambda'  : Item(status='D ', wc_rev='-', copied='+'),
+    'A/B2/F'       : Item(status='D ', wc_rev='-', copied='+'),
+    })
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
+  # Stuff copied from state deleted=true is now schedule=delete.
+  # Attempts to revert the schedule=delete will fail, but should not
+  # break the wc.  It's very important that the directory revert fails
+  # since it's a placeholder rather than a full hierarchy
+  out, err = svntest.main.run_svn(1, 'revert', '--recursive',
+                                  os.path.join(B2_path, 'F'))
+  for line in err:
+    if line.find("Error restoring text") != -1:
+      break
+  else:
+    raise svntest.Failure
+  out, err = svntest.main.run_svn(1, 'revert', os.path.join(B2_path, 'lambda'))
+  for line in err:
+    if line.find("Error restoring text") != -1:
+      break
+  else:
+    raise svntest.Failure
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
+  # Revert the entire copy including the schedule delete bits
+  svntest.actions.run_and_verify_svn(None, None, [], 'revert', '--recursive',
+                                     B2_path)
+  expected_status.remove('A/B2',
+                         'A/B2/E',
+                         'A/B2/E/beta',
+                         'A/B2/E/alpha',
+                         'A/B2/lambda',
+                         'A/B2/F')
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+  svntest.main.safe_rmtree(B2_path)
+
+  # Copy again and commit
+  svntest.actions.run_and_verify_svn(None, None, [], 'copy', B_path, B2_path)
+  expected_status.add({
+    'A/B2'        : Item(status='  ', wc_rev=3),
+    'A/B2/E'      : Item(status='  ', wc_rev=3),
+    'A/B2/E/beta' : Item(status='  ', wc_rev=3),
+    })
+  expected_output = svntest.wc.State(wc_dir, {
+    'A/B2'         : Item(verb='Adding'),
+    'A/B2/E/alpha' : Item(verb='Deleting'),
+    'A/B2/lambda'  : Item(verb='Deleting'),
+    'A/B2/F'       : Item(verb='Deleting'),
+    })
+  svntest.actions.run_and_verify_commit (wc_dir,
+                                         expected_output,
+                                         expected_status,
+                                         None, None, None, None, None,
+                                         wc_dir)
+
+#----------------------------------------------------------------------
+# Test for copy into a non-existent URL path 
+def url_to_non_existent_url_path(sbox):
+  "svn cp src-URL non-existent-URL-path"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  dirURL1 = svntest.main.current_repo_url + "/A/B/E"
+  dirURL2 = svntest.main.current_repo_url + "/G/C/E/I"
+
+  # Look for both possible versions of the error message, as the DAV
+  # error is worded differently from that of other RA layers.
+  msg = ".*: (Path 'G' not present|.*G' path not found)"
+
+  # Expect failure on 'svn cp SRC DST' where one or more ancestor
+  # directories of DST do not exist
+  out, err = svntest.main.run_svn(1,
+                                  'cp', dirURL1, dirURL2,
+                                  '--username', svntest.main.wc_author,
+                                  '--password', svntest.main.wc_passwd,
+                                  '-m', 'fooogle')
+  for err_line in err:
+    if re.match (msg, err_line):
+      break
+  else:
+    print "message \"" + msg + "\" not found in error output: ", err
+    raise svntest.Failure
+
+
+#----------------------------------------------------------------------
+# Test for a copying (URL to URL) an old rev of a deleted file in a
+# deleted directory.
+def non_existent_url_to_url(sbox):
+  "svn cp oldrev-of-deleted-URL URL"
+
+  sbox.build()
+
+  adg_url = svntest.main.current_repo_url + '/A/D/G'
+  pi_url = svntest.main.current_repo_url + '/A/D/G/pi'
+  new_url = svntest.main.current_repo_url + '/newfile'
+
+  svntest.actions.run_and_verify_svn(None, None, None, 'delete',
+                                     adg_url, '-m', '')
+
+  svntest.actions.run_and_verify_svn(None, None, None, 'copy',
+                                     '-r', '1', pi_url, new_url,
+                                     '-m', '')
+
+#----------------------------------------------------------------------
+def old_dir_url_to_url(sbox):
+  "test URL to URL copying edge case"
+
+  sbox.build()
+
+  adg_url = svntest.main.current_repo_url + '/A/D/G'
+  pi_url = svntest.main.current_repo_url + '/A/D/G/pi'
+  iota_url = svntest.main.current_repo_url + '/iota'
+  new_url = svntest.main.current_repo_url + '/newfile'
+
+  # Delete a directory
+  svntest.actions.run_and_verify_svn(None, None, None, 'delete',
+                                     adg_url, '-m', '')
+
+  # Copy a file to where the directory used to be
+  svntest.actions.run_and_verify_svn(None, None, None, 'copy',
+                                     iota_url, adg_url,
+                                     '-m', '')
+
+  # Try copying a file that was in the deleted directory that is now a
+  # file
+  svntest.actions.run_and_verify_svn(None, None, None, 'copy',
+                                     '-r', '1', pi_url, new_url,
+                                     '-m', '')
+
+
+
+#----------------------------------------------------------------------
+# Test fix for issue 2224 - copying wc dir to itself causes endless
+# recursion
+def wc_copy_dir_to_itself(sbox):
+  "copy wc dir to itself"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  dnames = ['A','A/B']
+
+  for dirname in dnames:
+    dir_path = os.path.join(sbox.wc_dir, dirname)
+
+    # try to copy dir to itself
+    svntest.actions.run_and_verify_svn(None, svntest.SVNAnyOutput, None,
+                                       'copy', dir_path, dir_path)
+
 ########################################################################
 # Run the tests
 
@@ -1383,6 +1590,11 @@ test_list = [ None,
               copy_over_missing_file,
               repos_to_wc_1634,
               double_uri_escaping_1814,
+              wc_to_wc_copy_deleted,
+              url_to_non_existent_url_path,
+              non_existent_url_to_url,
+              old_dir_url_to_url,
+              wc_copy_dir_to_itself,
              ]
 
 if __name__ == '__main__':
